<?php

namespace Teknisa\TCO\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Tco00600GerenteNegocioService extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $connection;
    protected $environment;
    protected $crudService;
    protected $entityManager;
    protected $dataSourceManager;

    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager) {
        $this->crudService       = $crudService;
        $this->connection        = $entityManager->getConnection();
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
    }

    /**
     * Função responsável por adicionar operadores
     * como gerentes caso estejam ativos e validos, e após
     * a adição checa a existência de erros no response 
     * retornados pelo 'parent::save' para exibir mensagem
     * customizada. 
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function saveManager(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'gerentenegocio';
        parent::save($request, $response);

        $responseError = $response->getError();        
        if(isset($responseError) && strpos($responseError->getMessage(), 'ORA-00001')){
            $response->setError(new Error('TCO_REGISTRO_EXISTE', 1));
        }
    }

    /**
     * Função que retorna o estado de atividade
     * do operador passado por parâmetro.
     * 
     * @param {String} $cdOperador Código do operador
     *  a ser testado.
     * 
     * @return {Boolean} Estado de atividade do 
     *  operador.
     */
    public function isActiveOperator($cdOperador){        
        $conditions = array(
            array(
                "columnName" => "CDOPERADOR", 
                "operator" => "=",
                "value" => $cdOperador
            )
        );

        $filterCriteria = new FilterCriteria("operador", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);

        return $dataSet->getRows()[0]['IDOPERATIVO']=='A';
    }

    /**
     * Função responsável por verificar se o dataset
     * de adição de um novo gerente é válido, verificando
     * se os parametros do newCode são válidos e se o
     * CDOPERADORLIDER está vazio.
     * 
     * @param {DTO/Row} $row Objeto $row contendo os 
     *  dados de inclusão do operador.
     * 
     * @return {Boolean} Validade do dataSet.
     */
    public function isValidNewManagerDataSet($row){
        $valid = true;
        $validParameters = array(
            "__table_pk"             => "NRGERENTENEGOCIO",
            "__using_unit"           => false,
            "__new_code_pk"          => "GERENTENEGOCIO",
            "__cast_number"          => false,
            "__new_code_length"      => 1,
            "__columns_to_return"    => array("NRGERENTENEGOCIO"),
            "__new_code_column_size" => 20
        );

        foreach($validParameters as $paramKey => $paramValue){
            if($row->get($paramKey)!=$paramValue)
                $valid = false;
        }

        return $valid && (($row->has('CDOPERADORLIDER') && $row->get('__is_new'))
        || (!$row->has('CDOPERADORLIDER') && !$row->get('__is_new')));
    }

    /**
     * Função responsável por remover gerentes.
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function removeManager(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'gerentenegocio';
        parent::delete($request, $response);
    }

    /**
     * Função responsável por retornar se operador registros
     * filhos em outras tabelas
     * 
     * @param {String} $nrGerenteNegocio Número do gerente
     *  de negócio.
     */
    public function getDependencies($nrGerenteNegocio){
        $sql = "SELECT 'TCO00600_GERNEGOCIO_USED_PROPOSTACOMERCIAL' AS MESSAGECODE,
                    CDPROPOSTACOMERCIAL AS PARAM1,
                    NULL AS PARAM2
                FROM PROPOSTACOMERCIAL
                WHERE NRGERENTENEGOCIO =:NRGERENTENEGOCIO
                
                UNION ALL
                SELECT 'TCO00600_GERNEGOCIO_USED_RESPONSAVELPROPCOM' AS MESSAGECODE,
                    PCOM.CDPROPOSTACOMERCIAL AS PARAM1,
                    NULL AS PARAM2
                FROM RESPONSAVELPROPCOM RPC
                    JOIN PROPOSTACOMERCIAL PCOM ON RPC.NRPROPOSTACOMERCIAL = PCOM.NRPROPOSTACOMERCIAL
                WHERE RPC.NRGERENTENEGOCIO = :NRGERENTENEGOCIO
                
                UNION ALL
                SELECT 'TCO00600_GERENTES_INVALIDOS_REMOVE' AS MESSAGECODE,
                    TO_CHAR(GN.CDOPERADOR) AS PARAM1,
                    OP.NMOPERADOR AS PARAM2
                FROM GERENTENEGOCIO GN
                    INNER JOIN OPERADOR OP ON OP.CDOPERADOR = GN.CDOPERADOR
                WHERE GN.CDOPERADORLIDER IN (
                        SELECT CDOPERADOR
                            FROM GERENTENEGOCIO
                            WHERE NRGERENTENEGOCIO =:NRGERENTENEGOCIO)";

        $params = array(
            "NRGERENTENEGOCIO" => $nrGerenteNegocio
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        return $result;
    }


    /**
     * Função responsável por adicionar gerentes à
     * liderança de outro.
     * 
     * @param {String} $cdLider Código do gerente
     *  correspondente.
     * @param {Array[String]} Array de operadores 
     *  a serem removidos.
     */
    public function addToManagerGrid($cdLider, $toAdd){
        $removedRows = $this->removeInvalidManagersToInclude($cdLider, $toAdd);
        $dataSetRows = $this->buildRowToUpdateManagerGrid($cdLider, $toAdd, 'A');
        $this->crudService->save('gerentenegocio', $dataSetRows);
        $dataSet = array();

        if(sizeof($removedRows)){
            $conditions = array(
                array(
                    "columnName" => "CDOPERADOR", 
                    "operator" => "IN",
                    "value" => $removedRows
                )
            );
    
            $filterCriteria = new FilterCriteria("operador", $conditions);
            $dataSet = array_map(function($val){
                return array(
                    "CDOPERADOR" => $val["CDOPERADOR"],
                    "NMOPERADOR" => $val["NMOPERADOR"]
                );
            }, $this->dataSourceManager->findBy($filterCriteria)->getRows());

        }
        return $dataSet;
    }

    /**
     * Função responsável por remover do set de 
     * gerentes, aqueles que são inválidos para inclusão
     * na liderança do referido lider.
     * A função evita a referência circular, evitando 
     * que um gerente seja líder de alguem acima dele.
     * 
     * @param {String} $cdLider Código do referido lider.
     * @param {Array[String]} $toRemove Lista de gerentes
     *  para inclusão.
     * 
     * @return {Array[String]} Array de códigos de operadores
     *  removidos.
     */
    private function removeInvalidManagersToInclude($cdLider, &$toRemove){
        $managerLeadTree = $this->getManagerLeadTree($cdLider);
        $removedRows = array();

        foreach($toRemove as $key => $manager){
            if(in_array($manager['CDOPERADOR'], $managerLeadTree)){
                $removedRows[] = $manager['CDOPERADOR'];
                unset($toRemove[$key]);
            }
        }

        return $removedRows;
    }
    
    /**
     * Função responsável por retornar a árvore de
     * liderança de um gerente, ou seja, retorna todos
     * que estão acima dele.
     * 
     * @param {String} $cdLider Código do referido lider.
     *
     * @return {Array[String]} Array de números de gerentes
     *  que estão acima do lider.
     */
    private function getManagerLeadTree($cdLider){
        $sql = "SELECT
                    CDOPERADOR
                FROM
                    GERENTENEGOCIO GN
                START WITH GN.CDOPERADOR = :CDOPERADOR
                CONNECT BY
                    GN.CDOPERADOR = PRIOR GN.CDOPERADORLIDER";

        $params = array(
            "CDOPERADOR" => $cdLider
        );

	    $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        $result = array_map(function($register){
            return $register['CDOPERADOR'];
        }, $result);

        return $result;
    }

    /**
     * Função responsável por remover gerentes da
     * liderança de outro.
     * 
     * @param {String} $cdLider Código do gerente
     *  correspondente.
     * @param {Array[String]} Array de códigos de
     *  operadores a serem removidos.
     */
    public function removeFromManagerGrid($cdLider, $toRemove){
        $dataSetRows = $this->buildRowToUpdateManagerGrid($cdLider, $toRemove, 'R');
        $this->crudService->save('gerentenegocio', $dataSetRows);
    }

    /**
     * Função responsável por criar um objeto DataSet
     * para atualizar os dados de lider dos gerentes
     * passados por parâmetros por meio do método save
     * do crud.
     * 
     * @param {String} $cdLider Código do lider 
     *  correspondente.
     * @param {Array[String]} $toUpdate Códigos dos
     *  gerentes a serem incluidos/removidos.
     * @param {String} $operation Tipo da operação,
     *  'A' para adição e 'R' para remoção.
     * 
     * @return {Array} Array de rows preparadas.
     */
    private function buildRowToUpdateManagerGrid($cdLider, $toUpdate, $operation){        
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");

        foreach($toUpdate as &$row){
            if (is_object($row))
                $row = (array) $row;
            
            $row['NRORG'] = (isset($row['NRORG']) && !empty($row['NRORG'])) ? 
                $row['NRORG'] : 
                $this->environment->getNrOrgTrab();

            $row['IDATIVO'] = isset($row['IDATIVO']) ? 
                ($row['IDATIVO'] ?: 'S') :
                'S';

            $row['DTULTATU'] = $currentTime;
            $row['NRORGULTATU'] = $this->environment->getNrOrgTrab();
            $row['CDOPERULTATU'] = $this->environment->getCdOperador();
            $row = $this->crudService->getOthersFields('gerentenegocio', array('NRGERENTENEGOCIO'), $row);
            $row['CDOPERADORLIDER'] = $operation == 'A' ? $cdLider : '';
        }

        return $toUpdate;
    }
}